import Two from "two.js";

const colorList = ['#f5222d', '#d4380d', '#d46b08', '#d48806', '#d4b106', '#7cb305', '#389e0d', '#08979c', '#096dd9', '#531dab']

export default function game({ el, center, raduis, onError, onSuccess, onBingo }) {

  // 是否游戏结束
  let isFinished = false;

  // 是否失败
  // eslint-disable-next-line no-unused-vars
  let isError = false;

  // 每个区域的旋转角度, 10 个轿厢, 分 20 等份
  const perAngle = 2 * Math.PI / 20 // 分20等份

  // 大转盘半径
  // const raduis = 150;

  // 转盘中心坐标
  // const center;

  // 轿厢 - 子弹
  const bullets = []
  
  // 子弹是否在射击状态, 如果是, 不能进行其他操作
  let isShooting = false
  
  // 子弹飞行速度, 因为是向上飞, y 值是不端减小的过程
  const speed = -0.01;

  // 目标轿厢列表, 该列表主要用来判断子弹是否击中目标
  const cageList = [];

  // 轿厢的初始旋转弧度
  let rotateAngle = 0
  
  // 轿厢的旋转速度 - 单位弧度
  const rotateSpeed = 0.01

  // 目标轿厢与子弹轿厢的映射字典
  const mntMap = {}

  // 初始化
  const two = new Two({
    fullscreen: true,
    autostart: true
  }).appendTo(el);

  // 绘制转盘
  const drawRoundAbout = () => {
    const { x, y } = center

    // 绘制扇形区域
    // 该区域对游戏实际上没有用, 主要是梳理逻辑
    const arcList = Array(10).fill().map((_, inx) => {
      const startAngle = 2 * inx * perAngle
      const endAngle = startAngle + perAngle
      const arc = two.makeArcSegment(x, y, 0, raduis, startAngle, endAngle)
      arc.fill = colorList[inx]
      arc.noStroke()

      // 绘制目标轿厢
      // 1、计算扇形边上的中心坐标
      const cx = x + Math.cos(startAngle + perAngle / 2) * raduis
      const cy = y + Math.sin(startAngle + perAngle / 2) * raduis

      // 2、轿厢的宽度, 等于圆周长的 1/20
      const w = Math.PI * 2 * raduis / 20 // 周长 / 20
      const rect = two.makeRectangle(cx, cy, w, w);
      rect.id = `mnt-${inx}`
      rect.fill = colorList[inx]
      rect.noStroke()
      rect.__$angle = startAngle  // 轿厢的初始弧度

      cageList.push(rect)

      return arc
    })

    return arcList
  }
  const roundAbout = drawRoundAbout()

  // 旋转
  const rotate = () => {
    rotateAngle += rotateSpeed
    roundAbout.forEach((x, inx) => {
      x.rotation = rotateAngle
            
      // 计算旋转后的轿厢坐标
      const cage = cageList[inx]
      const angle = cage.__$angle + rotateAngle + perAngle / 2
      const cx = center.x + Math.cos(angle) * raduis
      const cy = center.y + Math.sin(angle) * raduis
      cage.position = new Two.Vector(cx, cy)

      // 如果有对应的挂载物
      if (mntMap[cage.id]) {
        mntMap[cage.id].position = cage.position
      }
    })
  }

  // 绘制子弹
  const drawBullets = (x, y) => {
    const w = 20, h = 20;

    const list = Array(10).fill().map((_, inx) => {
      const rect = two.makeRectangle(x, y, w, h)
      rect.id = `bullet-${inx}`
      rect.fill = 'black'
      rect.noStroke()

      return rect
    })

    bullets.push(...list)
  }
  drawBullets(center.x, center.y + 400) // 400 是距离转盘中心的长度
  
  // 待发射子弹
  const clip = bullets.slice()

  // 挂载到轮盘
  const mountToCage = (targ, cage) => {
    targ.position = cage.position
    // 写入映射表
    mntMap[cage.id] = targ
  }
  
  // 当前第几个子弹
  let currentBullet = null;
  // 射击
  const shooting = () => {
    isShooting = true
    const { top } = currentBullet.getBoundingClientRect()
    currentBullet.position = new Two.Vector(currentBullet.position.x, top + speed)

    const hitted = isHit(currentBullet)
    if (hitted === 'error') {
      const t = setTimeout(() => {
        isError = true
        isFinished = true
        onError()
        clearTimeout(t)
      }, 0)
      return
    }

    if (hitted) {
      mountToCage(currentBullet, hitted)
      currentBullet = null;
      isShooting = false;

      const t = setTimeout(() => {
        if (!clip.length) {
          isFinished = true
          onSuccess()
        } else {
          onBingo()
        }
        clearTimeout(t)
      }, 0)
    }
  }

  // 是否击中
  // 如果击中, 则返回目标轿厢
  const isHit = (bullet) => {
    const rect2 = bullet.getBoundingClientRect();
    const x = rect2.left + rect2.width / 2
    const y = rect2.top + rect2.height / 2

    if (y < (center.y + raduis)) {
      return 'error';
    }

    const cage = cageList.filter((it) => {
      const rect1 = it.getBoundingClientRect();

      // 如果当前子弹的中心点位于目标轿厢矩形范围内
      // 则代表击中
      return x >= rect1.left && x <= rect1.right &&
          y >= rect1.top && y <= rect1.bottom;
    })[0]

    if (!cage) return false;

    return !cage ? false : cage
  }
  
  // 重复绘制内容
  two.bind('update', () => {
    if (!isFinished) {
      rotate()
  
      if (currentBullet) {
        shooting()
      }
    }
  })

  // 绑定 dom click 事件 触发子弹发射
  el.addEventListener('click', () => {
    if (!isFinished) {
      if (clip.length && !isShooting) {
        currentBullet = clip.pop()
      }
    }
  })

  return () => {
    two.unbind('update');
    two.pause();
    el.removeChild(two.renderer.domElement);
  }
}
